/*
 * Evaluation.java
 *
 * Copyright (c) 2011 Daniel Wayne Crabtree - All Rights Reserved
 * 
 * If you use this, please cite my PhD Thesis.
 
   @PHDTHESIS{IWS11,
       AUTHOR =  {Daniel Crabtree},
       TITLE =   {Techniques for Improving Web Search by Understanding Queries},
       SCHOOL =  {School of Engineering and Computer Science, Victoria University of Wellington},
       ADDRESS = {Wellington, New Zealand},
       MONTH =   {May},    
       YEAR =    {2011}
   }
  
 */

package evaluation;

import qc4.*;

/*
 * This file demonstrates how to use the QC4 evaluation code to evaluate 
 * clusterings using an ideal clustering. 
 * 
 * Specifically, it constructs the clusterings and ideal clusterings used in 
 * the synthetic tests from chapter 4 of my PhD thesis, evaluates those 
 * clusterings using QC4, and outputs the results to System.out.
 * 
 * GetIdealClustering_Outlier and Test_Outlier demonstrate how to specify an 
 * outlier topic.
 */
class Evaluation
{       
    static void evaluateUsingQC4(String name, IdealClustering idealClustering, Clustering<Cluster> clustering)
    {        
        QC4 qc4 = new QC4(idealClustering, 3);
        System.out.println(name + " AQ : " + qc4.AQ(clustering));
        System.out.println(name + " WQ : " + qc4.WQ(clustering));
        System.out.println(name + " AC : " + qc4.AC(clustering));
        System.out.println(name + " WC : " + qc4.WC(clustering));
    }
    
    static IdealClustering GetIdealClustering_4_18()
    {
        ResultSet resultSet = new ResultSet(1100);
                
        IdealCluster green = new IdealCluster(resultSet);
        green.addPageRange(0, 1000);
        
        IdealCluster blue = new IdealCluster(resultSet);
        blue.addPageRange(1000, 1025);
        
        IdealCluster purple = new IdealCluster(resultSet);
        purple.addPageRange(1025, 1050);
        
        IdealCluster red = new IdealCluster(resultSet);
        red.addPageRange(1050, 1075);
        
        IdealCluster yellow = new IdealCluster(resultSet);
        yellow.addPageRange(1075, 1100);
        
        IdealClustering idealClustering_4_18 = new IdealClustering(resultSet);  
        idealClustering_4_18.add(green);
        idealClustering_4_18.add(blue);
        idealClustering_4_18.add(purple);
        idealClustering_4_18.add(red);
        idealClustering_4_18.add(yellow);
        idealClustering_4_18.setupRegions();
        
        return idealClustering_4_18;
    }
    
    static IdealClustering GetIdealClustering_4_24()
    {
        ResultSet resultSet = new ResultSet(125);
                
        IdealCluster green = new IdealCluster(resultSet);
        green.addPageRange(0, 25);
        
        IdealCluster blue = new IdealCluster(resultSet);
        blue.addPageRange(25, 50);
        
        IdealCluster purple = new IdealCluster(resultSet);
        purple.addPageRange(50, 75);
        
        IdealCluster red = new IdealCluster(resultSet);
        red.addPageRange(75, 100);
        
        IdealCluster yellow = new IdealCluster(resultSet);
        yellow.addPageRange(100, 125);
        
        IdealClustering idealClustering_4_24 = new IdealClustering(resultSet);  
        idealClustering_4_24.add(green);
        idealClustering_4_24.add(blue);
        idealClustering_4_24.add(purple);
        idealClustering_4_24.add(red);
        idealClustering_4_24.add(yellow);
        idealClustering_4_24.setupRegions();
        
        return idealClustering_4_24;
    }
    
    static IdealClustering GetIdealClustering_T_4_6()
    {
        ResultSet resultSet = new ResultSet(3000);
                
        IdealCluster blue = new IdealCluster(resultSet);
        blue.addPageRange(0, 1000);
        
        IdealCluster green = new IdealCluster(resultSet);
        green.addPageRange(1000, 2000);
        
        IdealCluster purple = new IdealCluster(resultSet);
        purple.addPageRange(2000, 3000);
                
        IdealClustering idealClustering_T_4_6 = new IdealClustering(resultSet);  
        idealClustering_T_4_6.add(green);
        idealClustering_T_4_6.add(blue);
        idealClustering_T_4_6.add(purple);
        idealClustering_T_4_6.setupRegions();
        
        return idealClustering_T_4_6;
    }
        
    static IdealClustering GetIdealClustering_4_27()
    {
        ResultSet resultSet = new ResultSet(180);
                
        IdealCluster green = new IdealCluster(resultSet);
        green.addPageRange(0, 100);
        
        IdealCluster blue = new IdealCluster(resultSet);
        blue.addPageRange(80, 180);
                
        IdealClustering idealClustering_4_27 = new IdealClustering(resultSet);  
        idealClustering_4_27.add(green);
        idealClustering_4_27.add(blue);
        idealClustering_4_27.setupRegions();
        
        return idealClustering_4_27;
    }
    
    static IdealClustering GetIdealClustering_4_29()
    {
        ResultSet resultSet = new ResultSet(1000);       
        
        IdealCluster red = new IdealCluster(resultSet, 1);
        red.addPageRange(0, 700);
               
        IdealCluster green = new IdealCluster(resultSet, 2);
        green.addPageRange(0, 400);
        
        IdealCluster blue = new IdealCluster(resultSet, 2);
        blue.addPageRange(400, 600);
        
        IdealCluster purple = new IdealCluster(resultSet, 2);
        purple.addPageRange(600, 700);
        
        red.addChild(green);
        red.addChild(blue);
        red.addChild(purple);
        
        IdealCluster yellow = new IdealCluster(resultSet, 1);
        yellow.addPageRange(700, 1000);
        
        IdealCluster guppieGreen = new IdealCluster(resultSet, 2);
        guppieGreen.addPageRange(700, 850);
        
        IdealCluster aqua = new IdealCluster(resultSet, 2);
        aqua.addPageRange(850, 940);
        
        IdealCluster pink = new IdealCluster(resultSet, 2);
        pink.addPageRange(940, 1000);
        
        yellow.addChild(guppieGreen);
        yellow.addChild(aqua);
        yellow.addChild(pink);
        
        IdealClustering idealClustering_4_29 = new IdealClustering(resultSet);
        idealClustering_4_29.add(red);
        idealClustering_4_29.add(yellow);
        idealClustering_4_29.add(green);
        idealClustering_4_29.add(blue);
        idealClustering_4_29.add(purple);
        idealClustering_4_29.add(guppieGreen);
        idealClustering_4_29.add(aqua);
        idealClustering_4_29.add(pink);
        idealClustering_4_29.setupRegions();
        
        return idealClustering_4_29;
    }
    
    static IdealClustering GetIdealClustering_Outlier()
    {
        ResultSet resultSet = new ResultSet(100);        
       
        IdealCluster green = new IdealCluster(resultSet);
        green.addPageRange(0, 50);
        
        IdealCluster red = new IdealCluster(resultSet);
        red.addPageRange(50, 80);
        
        IdealCluster outlier = new IdealCluster(resultSet);
        outlier.addPageRange(80, 100);
        outlier.setOutlier();
        
        IdealClustering idealClustering_Outlier = new IdealClustering(resultSet);
        idealClustering_Outlier.add(green);
        idealClustering_Outlier.add(red);
        idealClustering_Outlier.add(outlier);
        idealClustering_Outlier.setupRegions();
        
        return idealClustering_Outlier;
    }
    
    static void Test_4_19()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Cluster blue10 = new Cluster(resultSet);
        blue10.addPageRange(1000, 1010);
        
        Cluster blue5 = new Cluster(resultSet);
        blue5.addPageRange(1010, 1015);
        
        Cluster blue5purple5 = new Cluster(resultSet);
        blue5purple5.addPageRange(1010, 1015);
        blue5purple5.addPageRange(1025, 1030);
        
        Cluster blue25 = new Cluster(resultSet);
        blue25.addPageRange(1000, 1025);
        
        Cluster purple5 = new Cluster(resultSet);
        purple5.addPageRange(1030, 1035);
        
        Clustering<Cluster> clustering_4_19_A = new Clustering<Cluster>(resultSet);
        clustering_4_19_A.add(blue10);
        Evaluation.evaluateUsingQC4("4.19.A", idealClustering_4_18, clustering_4_19_A);
        
        Clustering<Cluster> clustering_4_19_B = new Clustering<Cluster>(resultSet);
        clustering_4_19_B.add(blue10);
        clustering_4_19_B.add(blue5);
        Evaluation.evaluateUsingQC4("4.19.B", idealClustering_4_18, clustering_4_19_B);
        
        Clustering<Cluster> clustering_4_19_C = new Clustering<Cluster>(resultSet);
        clustering_4_19_C.add(blue10);
        clustering_4_19_C.add(purple5);
        Evaluation.evaluateUsingQC4("4.19.C", idealClustering_4_18, clustering_4_19_C);
        
        Clustering<Cluster> clustering_4_19_D = new Clustering<Cluster>(resultSet);
        clustering_4_19_D.add(blue10);
        clustering_4_19_D.add(purple5);
        clustering_4_19_D.add(blue5purple5);        
        Evaluation.evaluateUsingQC4("4.19.D", idealClustering_4_18, clustering_4_19_D);
        
        Clustering<Cluster> clustering_4_19_E = new Clustering<Cluster>(resultSet);
        clustering_4_19_E.add(blue25);
        Evaluation.evaluateUsingQC4("4.19.E", idealClustering_4_18, clustering_4_19_E);
    }
    
    static void Test_4_20()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Cluster green920 = new Cluster(resultSet);
        green920.addPageRange(0, 920);
        
        Cluster green900 = new Cluster(resultSet);
        green900.addPageRange(0, 900);
        
        Cluster blue5 = new Cluster(resultSet);
        blue5.addPageRange(1000, 1005);
        
        Cluster purple5 = new Cluster(resultSet);
        purple5.addPageRange(1025, 1030);
        
        Cluster red5 = new Cluster(resultSet);
        red5.addPageRange(1050, 1055);
        
        Cluster yellow5 = new Cluster(resultSet);
        yellow5.addPageRange(1075, 1080);
        
        Cluster green107 = new Cluster(resultSet);
        green107.addPageRange(0, 107);
        
        Cluster blue10 = new Cluster(resultSet);
        blue10.addPageRange(1000, 1010);
        
        Cluster purple10 = new Cluster(resultSet);
        purple10.addPageRange(1025, 1035);
        
        Cluster red10 = new Cluster(resultSet);
        red10.addPageRange(1050, 1060);
        
        Cluster yellow10 = new Cluster(resultSet);
        yellow10.addPageRange(1075, 1085);
        
        Clustering<Cluster> clustering_4_20_A = new Clustering<Cluster>(resultSet);
        clustering_4_20_A.add(green920);
        Evaluation.evaluateUsingQC4("4.20.A", idealClustering_4_18, clustering_4_20_A);
        
        Clustering<Cluster> clustering_4_20_B = new Clustering<Cluster>(resultSet);
        clustering_4_20_B.add(green900);
        clustering_4_20_B.add(blue5);
        clustering_4_20_B.add(purple5);
        clustering_4_20_B.add(red5);
        clustering_4_20_B.add(yellow5);
        Evaluation.evaluateUsingQC4("4.20.B", idealClustering_4_18, clustering_4_20_B);
      
        // Setup Clustering for Test5
        Clustering<Cluster> clustering_4_20_C = new Clustering<Cluster>(resultSet);
        clustering_4_20_C.add(green107);
        clustering_4_20_C.add(blue10);
        clustering_4_20_C.add(purple10);
        clustering_4_20_C.add(red10);
        clustering_4_20_C.add(yellow10);
        Evaluation.evaluateUsingQC4("4.20.C", idealClustering_4_18, clustering_4_20_C);
    }
    
    static void Test_4_21()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Cluster green1000 = new Cluster(resultSet);
        green1000.addPageRange(0, 1000);
        
        Cluster blue25 = new Cluster(resultSet);
        blue25.addPageRange(1000, 1025);
        
        Cluster purple25 = new Cluster(resultSet);
        purple25.addPageRange(1025, 1050);
        
        Cluster red25 = new Cluster(resultSet);
        red25.addPageRange(1050, 1075);
        
        Cluster yellow25 = new Cluster(resultSet);
        yellow25.addPageRange(1075, 1100);
        
        Cluster blue25purple15red15 = new Cluster(resultSet);
        blue25purple15red15.addPageRange(1000, 1025);
        blue25purple15red15.addPageRange(1025, 1040);
        blue25purple15red15.addPageRange(1050, 1065);
        
        Clustering<Cluster> clustering_4_21_A = new Clustering<Cluster>(resultSet);
        clustering_4_21_A.add(green1000);
        clustering_4_21_A.add(blue25);
        clustering_4_21_A.add(purple25);
        clustering_4_21_A.add(red25);
        clustering_4_21_A.add(yellow25);      
        Evaluation.evaluateUsingQC4("4.21.A", idealClustering_4_18, clustering_4_21_A);
         
        Clustering<Cluster> clustering_4_21_B = new Clustering<Cluster>(resultSet);
        clustering_4_21_B.add(green1000);
        clustering_4_21_B.add(blue25);
        clustering_4_21_B.add(purple25);
        clustering_4_21_B.add(red25);
        clustering_4_21_B.add(yellow25);
        clustering_4_21_B.add(blue25purple15red15);
        Evaluation.evaluateUsingQC4("4.21.B", idealClustering_4_18, clustering_4_21_B);
        
        Clustering<Cluster> clustering_4_21_C = new Clustering<Cluster>(resultSet);
        clustering_4_21_C.add(blue25);
        Evaluation.evaluateUsingQC4("4.21.C", idealClustering_4_18, clustering_4_21_C);
    }
    
    static void Test_4_22()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Clustering<Cluster> clustering_4_22_Singleton = new Clustering<Cluster>(resultSet);
        for(int i = 0; i < 1100; i++)
        {
            Cluster singleton = new Cluster(resultSet);
            singleton.addPage(i);
            clustering_4_22_Singleton.add(singleton);
        }
        Evaluation.evaluateUsingQC4("4.22.Singleton", idealClustering_4_18, clustering_4_22_Singleton);
        
        Cluster giant = new Cluster(resultSet);
        giant.addPageRange(0, 1100);

        Clustering<Cluster> clustering_4_22_Giant = new Clustering<Cluster>(resultSet);
        clustering_4_22_Giant.add(giant);
        Evaluation.evaluateUsingQC4("4.22.Giant", idealClustering_4_18, clustering_4_22_Giant);
        
        Cluster random1 = new Cluster(resultSet);
        random1.addPageRange(0, 170);
        random1.addPageRange(1000, 1005);
        random1.addPageRange(1025, 1030);
        random1.addPageRange(1050, 1055);
        random1.addPageRange(1075, 1080);
        
        Cluster random2 = new Cluster(resultSet);
        random2.addPageRange(170, 340);
        random2.addPageRange(1005, 1010);
        random2.addPageRange(1030, 1035);
        random2.addPageRange(1055, 1060);
        random2.addPageRange(1080, 1085);
        
        Cluster random3 = new Cluster(resultSet);
        random3.addPageRange(340, 510);
        random3.addPageRange(1010, 1015);
        random3.addPageRange(1035, 1040);
        random3.addPageRange(1060, 1065);
        random3.addPageRange(1085, 1090);
        
        Cluster random4 = new Cluster(resultSet);
        random4.addPageRange(510, 680);
        random4.addPageRange(1015, 1020);
        random4.addPageRange(1040, 1045);
        random4.addPageRange(1065, 1070);
        random4.addPageRange(1090, 1095);
        
        Cluster random5 = new Cluster(resultSet);
        random5.addPageRange(680, 850);
        random5.addPageRange(1020, 1025);
        random5.addPageRange(1045, 1050);
        random5.addPageRange(1070, 1075);
        random5.addPageRange(1095, 1100);
        
        Clustering<Cluster> clustering_4_22_Random = new Clustering<Cluster>(resultSet);
        clustering_4_22_Random.add(random1);
        clustering_4_22_Random.add(random2);
        clustering_4_22_Random.add(random3);
        clustering_4_22_Random.add(random4);
        clustering_4_22_Random.add(random5);
        Evaluation.evaluateUsingQC4("4.22.Random", idealClustering_4_18, clustering_4_22_Random);
    }
    
    static void Test_4_23()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Cluster green50yellow10 = new Cluster(resultSet);       
        green50yellow10.addPageRange(0, 50);
        green50yellow10.addPageRange(1090, 1100);
        
        Cluster blue20purple10 = new Cluster(resultSet);
        blue20purple10.addPageRange(1000, 1020);
        blue20purple10.addPageRange(1025, 1035);
        
        Cluster green40purple10red15 = new Cluster(resultSet);
        green40purple10red15.addPageRange(100, 140);
        green40purple10red15.addPageRange(1035, 1045);
        green40purple10red15.addPageRange(1050, 1065);
                
        Cluster blue5red5yellow10 = new Cluster(resultSet);
        blue5red5yellow10.addPageRange(1020, 1025);
        blue5red5yellow10.addPageRange(1065, 1070);
        blue5red5yellow10.addPageRange(1075, 1085);
        
        Clustering<Cluster> clustering_4_23_A = new Clustering<Cluster>(resultSet);
        clustering_4_23_A.add(green50yellow10);
        clustering_4_23_A.add(blue20purple10);
        clustering_4_23_A.add(green40purple10red15);
        clustering_4_23_A.add(blue5red5yellow10);
        Evaluation.evaluateUsingQC4("4.23.A", idealClustering_4_18, clustering_4_23_A);
        
        Cluster green50 = new Cluster(resultSet);
        green50.addPageRange(0, 50);
               
        Cluster green40 = new Cluster(resultSet);
        green40.addPageRange(100, 140);
        
        Cluster blue5 = new Cluster(resultSet);
        blue5.addPageRange(1005, 1010);
        
        Cluster purple4 = new Cluster(resultSet);
        purple4.addPageRange(1025, 1029);
        
        Cluster red5 = new Cluster(resultSet);
        red5.addPageRange(1050, 1055);
        
        Cluster yellow4 = new Cluster(resultSet);
        yellow4.addPageRange(1075, 1079);
        
        Clustering<Cluster> clustering_4_23_B = new Clustering<Cluster>(resultSet);
        clustering_4_23_B.add(green50);
        clustering_4_23_B.add(green40);
        clustering_4_23_B.add(blue5);
        clustering_4_23_B.add(purple4);
        clustering_4_23_B.add(red5);
        clustering_4_23_B.add(yellow4);
        Evaluation.evaluateUsingQC4("4.23.B", idealClustering_4_18, clustering_4_23_B);   
    }
       
    static void Test_4_25()
    {
        IdealClustering idealClustering_4_24 = Evaluation.GetIdealClustering_4_24();
        ResultSet resultSet = idealClustering_4_24.getResultSet();
        
        Cluster blue25red12 = new Cluster(resultSet);
        blue25red12.addPageRange(25, 50);
        blue25red12.addPageRange(75, 87);
        
        Cluster blue25red6yellow6 = new Cluster(resultSet);
        blue25red6yellow6.addPageRange(25, 50);
        blue25red6yellow6.addPageRange(75, 81);
        blue25red6yellow6.addPageRange(100, 106);
        
        Cluster blue25red3 = new Cluster(resultSet);
        blue25red3.addPageRange(25, 50);
        blue25red3.addPageRange(75, 78);
                       
        Clustering<Cluster> clustering_4_25_A = new Clustering<Cluster>(resultSet);
        clustering_4_25_A.add(blue25red12);
        Evaluation.evaluateUsingQC4("4.25.A", idealClustering_4_24, clustering_4_25_A);
        
        Clustering<Cluster> clustering_4_25_B = new Clustering<Cluster>(resultSet);
        clustering_4_25_B.add(blue25red6yellow6);
        Evaluation.evaluateUsingQC4("4.25.B", idealClustering_4_24, clustering_4_25_B);
        
        Clustering<Cluster> clustering_4_25_C = new Clustering<Cluster>(resultSet);
        clustering_4_25_C.add(blue25red3);
        Evaluation.evaluateUsingQC4("4.25.C", idealClustering_4_24, clustering_4_25_C);
    }

    static void Test_4_26()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Cluster green900 = new Cluster(resultSet);
        green900.addPageRange(0, 900);

        Cluster green450 = new Cluster(resultSet);
        green450.addPageRange(0, 450);
        
        Cluster green450_2 = new Cluster(resultSet);
        green450_2.addPageRange(450, 900);       
        
        // Setup Clustering for Test3
        Clustering<Cluster> clustering_4_26_A = new Clustering<Cluster>(resultSet);
        clustering_4_26_A.add(green900);        
        Evaluation.evaluateUsingQC4("4.26.A", idealClustering_4_18, clustering_4_26_A);
        
        // Setup Clustering for Test9
        Clustering<Cluster> clustering_4_25_B = new Clustering<Cluster>(resultSet);
        clustering_4_25_B.add(green450);
        clustering_4_25_B.add(green450_2);
        Evaluation.evaluateUsingQC4("4.26.B", idealClustering_4_18, clustering_4_25_B);
    }
       
    static void Test_4_8()
    {
        IdealClustering idealClustering_T_4_6 = Evaluation.GetIdealClustering_T_4_6();
        ResultSet resultSet = idealClustering_T_4_6.getResultSet();
       
        Cluster blue1000 = new Cluster(resultSet);
        blue1000.addPageRange(0, 1000);
        
        Cluster green1000 = new Cluster(resultSet);
        green1000.addPageRange(1000, 2000);
        
        Cluster purple1000 = new Cluster(resultSet);
        purple1000.addPageRange(2000, 3000);
        
        Cluster blue700 = new Cluster(resultSet);
        blue700.addPageRange(0, 700);
        
        Cluster blue200 = new Cluster(resultSet);
        blue200.addPageRange(700, 900);
        
        Clustering<Cluster> clustering_4_8_LHS = new Clustering<Cluster>(resultSet);
        clustering_4_8_LHS.add(blue1000);
        clustering_4_8_LHS.add(green1000);
        clustering_4_8_LHS.add(purple1000);
        Evaluation.evaluateUsingQC4("4.8.LHS", idealClustering_T_4_6, clustering_4_8_LHS);
        
        Clustering<Cluster> clustering_4_8_RHS = new Clustering<Cluster>(resultSet);
        clustering_4_8_RHS.add(blue700);
        clustering_4_8_RHS.add(blue200);
        Evaluation.generateClusters(4, 25, 900, clustering_4_8_RHS, resultSet);   //   4x blue25        
        Evaluation.generateClusters(41, 20, 1000, clustering_4_8_RHS, resultSet); //  41x green20
        Evaluation.generateClusters(45, 4, 1820, clustering_4_8_RHS, resultSet);  //  45x green4
        Evaluation.generateClusters(9, 20, 2000, clustering_4_8_RHS, resultSet);  //   9x purple20
        Evaluation.generateClusters(205, 4, 2180, clustering_4_8_RHS, resultSet); // 205x purple4
        Evaluation.evaluateUsingQC4("4.8.RHS", idealClustering_T_4_6, clustering_4_8_RHS);
        
        Clustering<Cluster> clustering_4_8_LHS_Blue = new Clustering<Cluster>(resultSet);
        clustering_4_8_LHS_Blue.add(blue1000);
        Evaluation.evaluateUsingQC4("4.8.LHS.Blue", idealClustering_T_4_6, clustering_4_8_LHS_Blue);
        
        Clustering<Cluster> clustering_4_8_RHS_Blue = new Clustering<Cluster>(resultSet);
        clustering_4_8_RHS_Blue.add(blue700);
        clustering_4_8_RHS_Blue.add(blue200);
        Evaluation.generateClusters(4, 25, 900, clustering_4_8_RHS_Blue, resultSet); // 4x blue25
        Evaluation.evaluateUsingQC4("4.8.RHS.Blue", idealClustering_T_4_6, clustering_4_8_RHS_Blue);
        
        Clustering<Cluster> clustering_4_8_RHS_Green = new Clustering<Cluster>(resultSet);     
        Evaluation.generateClusters(41, 20, 1000, clustering_4_8_RHS_Green, resultSet); // 41x green20
        Evaluation.generateClusters(45, 4, 1820, clustering_4_8_RHS_Green, resultSet);  // 45x green4
        Evaluation.evaluateUsingQC4("4.8.RHS.Green", idealClustering_T_4_6, clustering_4_8_RHS_Green);
        
        Clustering<Cluster> clustering_4_8_RHS_Purple = new Clustering<Cluster>(resultSet);
        Evaluation.generateClusters(9, 20, 2000, clustering_4_8_RHS_Purple, resultSet);  //   9x purple20
        Evaluation.generateClusters(205, 4, 2180, clustering_4_8_RHS_Purple, resultSet); // 205x purple4
        Evaluation.evaluateUsingQC4("4.8.RHS.Purple", idealClustering_T_4_6, clustering_4_8_RHS_Purple);
    }
    private static void generateClusters(int number, int size, int start, Clustering<Cluster> clustering, ResultSet resultSet)
    {
        Cluster cluster = null;
        for(int pageLocation = start; pageLocation < start + number * size; pageLocation++)
        {
            if(pageLocation % size == 0)
            {
                if(cluster != null)
                    clustering.add(cluster);

                cluster = new Cluster(resultSet);
            }
            cluster.addPage(pageLocation);
        }
        clustering.add(cluster);
    }

    static void Test_4_28()
    {
        IdealClustering idealClustering_4_27 = Evaluation.GetIdealClustering_4_27();
        ResultSet resultSet = idealClustering_4_27.getResultSet();
               
        Cluster green100 = new Cluster(resultSet);
        green100.addPageRange(0, 100);
        
        Cluster blue100 = new Cluster(resultSet);
        blue100.addPageRange(80, 180);
        
        Cluster green80 = new Cluster(resultSet);
        green80.addPageRange(0, 80);
        
        Cluster blue80 = new Cluster(resultSet);
        blue80.addPageRange(100, 180);
              
        Clustering<Cluster> clustering_4_28_A = new Clustering<Cluster>(resultSet);
        clustering_4_28_A.add(green100);
        clustering_4_28_A.add(blue100);
        Evaluation.evaluateUsingQC4("4.28.A", idealClustering_4_27, clustering_4_28_A);
        
        Clustering<Cluster> clustering_4_28_B = new Clustering<Cluster>(resultSet);
        clustering_4_28_B.add(green80);
        clustering_4_28_B.add(blue100);
        Evaluation.evaluateUsingQC4("4.28.B", idealClustering_4_27, clustering_4_28_B);
        
        Clustering<Cluster> clustering_4_28_C = new Clustering<Cluster>(resultSet);
        clustering_4_28_C.add(green80);
        clustering_4_28_C.add(blue80);
        Evaluation.evaluateUsingQC4("4.28.C", idealClustering_4_27, clustering_4_28_C);
    }
    
    static void Test_4_30()
    {
        IdealClustering idealClustering_4_29 = Evaluation.GetIdealClustering_4_29();
        ResultSet resultSet = idealClustering_4_29.getResultSet();
                
        Cluster green400 = new Cluster(resultSet);
        green400.addPageRange(0, 400);
        
        Cluster blue200 = new Cluster(resultSet);
        blue200.addPageRange(400, 600);
        
        Cluster guppieGreen150 = new Cluster(resultSet);
        guppieGreen150.addPageRange(700, 850);
                
        Cluster pink60 = new Cluster(resultSet);
        pink60.addPageRange(940, 1000);
        
        Cluster green150blue150 = new Cluster(resultSet);
        green150blue150.addPageRange(250, 400);
        green150blue150.addPageRange(400, 450);
               
        Clustering<Cluster> clustering_4_30_A = new Clustering<Cluster>(resultSet);
        clustering_4_30_A.add(green150blue150);
        Evaluation.evaluateUsingQC4("4.30.A", idealClustering_4_29, clustering_4_30_A);
        
        Clustering<Cluster> clustering_4_30_B = new Clustering<Cluster>(resultSet);
        clustering_4_30_B.add(green400);
        clustering_4_30_B.add(blue200);
        clustering_4_30_B.add(guppieGreen150);
        clustering_4_30_B.add(pink60);
        Evaluation.evaluateUsingQC4("4.30.B", idealClustering_4_29, clustering_4_30_B);
    }
    
    static void Test_4_31()
    {
        IdealClustering idealClustering_4_29 = Evaluation.GetIdealClustering_4_29();
        ResultSet resultSet = idealClustering_4_29.getResultSet();
        
        Cluster red700 = new Cluster(resultSet);
        red700.addPageRange(0, 700);
        
        Cluster green400 = new Cluster(resultSet);
        green400.addPageRange(0, 400);
        
        Cluster blue200 = new Cluster(resultSet);
        blue200.addPageRange(400, 600);
        
        Cluster purple100 = new Cluster(resultSet);
        purple100.addPageRange(600, 700);
        
        Cluster yellow300 = new Cluster(resultSet);
        yellow300.addPageRange(700, 1000);
        
        Cluster guppieGreen150 = new Cluster(resultSet);
        guppieGreen150.addPageRange(700, 850);
        
        Cluster aqua90 = new Cluster(resultSet);
        aqua90.addPageRange(850, 940);
        
        Cluster pink60 = new Cluster(resultSet);
        pink60.addPageRange(940, 1000);
        
        Clustering<Cluster> clustering_4_31_A = new Clustering<Cluster>(resultSet);
        clustering_4_31_A.add(red700);
        clustering_4_31_A.add(yellow300);
        Evaluation.evaluateUsingQC4("4.31.A", idealClustering_4_29, clustering_4_31_A);

        Clustering<Cluster> clustering_4_31_B = new Clustering<Cluster>(resultSet);
        clustering_4_31_B.add(green400);
        clustering_4_31_B.add(blue200);
        clustering_4_31_B.add(purple100);
        clustering_4_31_B.add(guppieGreen150);
        clustering_4_31_B.add(aqua90);
        clustering_4_31_B.add(pink60);
        Evaluation.evaluateUsingQC4("4.31.B", idealClustering_4_29, clustering_4_31_B);
        
        Clustering<Cluster> clustering_4_31_C = new Clustering<Cluster>(resultSet);
        clustering_4_31_C.add(red700);
        clustering_4_31_C.add(guppieGreen150);
        clustering_4_31_C.add(aqua90);
        clustering_4_31_C.add(pink60);
        Evaluation.evaluateUsingQC4("4.31.C", idealClustering_4_29, clustering_4_31_C);
        
        Clustering<Cluster> clustering_4_31_D = new Clustering<Cluster>(resultSet);
        clustering_4_31_D.add(red700);
        clustering_4_31_D.add(green400);
        clustering_4_31_D.add(blue200);
        clustering_4_31_D.add(purple100);
        clustering_4_31_D.add(guppieGreen150);
        clustering_4_31_D.add(aqua90);
        clustering_4_31_D.add(pink60);
        Evaluation.evaluateUsingQC4("4.31.D", idealClustering_4_29, clustering_4_31_D);
        
    }
    
    static void Test_4_32()
    {
        IdealClustering idealClustering_4_18 = Evaluation.GetIdealClustering_4_18();
        ResultSet resultSet = idealClustering_4_18.getResultSet();
        
        Cluster green20 = new Cluster(resultSet);
        green20.addPageRange(0, 20);
        
        Cluster green40 = new Cluster(resultSet);
        green40.addPageRange(0, 40);
        
        Cluster green150 = new Cluster(resultSet);
        green150.addPageRange(0, 150);
        
        Cluster green300 = new Cluster(resultSet);
        green300.addPageRange(0, 300);
        
        Clustering<Cluster> clustering_4_32_A = new Clustering<Cluster>(resultSet);
        clustering_4_32_A.add(green20);
        Evaluation.evaluateUsingQC4("4.32.A", idealClustering_4_18, clustering_4_32_A);
        
        Clustering<Cluster> clustering_4_32_B = new Clustering<Cluster>(resultSet);
        clustering_4_32_B.add(green40);
        Evaluation.evaluateUsingQC4("4.32.B", idealClustering_4_18, clustering_4_32_B);
        
        Clustering<Cluster> clustering_4_32_C = new Clustering<Cluster>(resultSet);
        clustering_4_32_C.add(green150);
        Evaluation.evaluateUsingQC4("4.32.C", idealClustering_4_18, clustering_4_32_C);
        
        Clustering<Cluster> clustering_4_32_D = new Clustering<Cluster>(resultSet);
        clustering_4_32_D.add(green300);
        Evaluation.evaluateUsingQC4("4.32.D", idealClustering_4_18, clustering_4_32_D);
    }
    
    static void Test_Outlier()
    {
        IdealClustering idealClustering_Outlier = Evaluation.GetIdealClustering_Outlier();
        ResultSet resultSet = idealClustering_Outlier.getResultSet();
                
        Cluster green20outlier5 = new Cluster(resultSet);
        green20outlier5.addPageRange(0, 20);
        green20outlier5.addPageRange(80, 85);
        
        Cluster red10 = new Cluster(resultSet);
        red10.addPageRange(50, 60);
        
        Cluster outlier15red5 = new Cluster(resultSet);
        outlier15red5.addPageRange(80, 95);
        outlier15red5.addPageRange(60, 65);
               
        Clustering<Cluster> clustering_Outlier_1 = new Clustering<Cluster>(resultSet);
        clustering_Outlier_1.add(green20outlier5);
        clustering_Outlier_1.add(red10);
        clustering_Outlier_1.add(outlier15red5);
        Evaluation.evaluateUsingQC4("Outlier.1", idealClustering_Outlier, clustering_Outlier_1);
        
        Clustering<Cluster> clustering_Outlier_2 = new Clustering<Cluster>(resultSet);
        clustering_Outlier_2.add(green20outlier5);
        clustering_Outlier_2.add(red10);
        Evaluation.evaluateUsingQC4("Outlier.2", idealClustering_Outlier, clustering_Outlier_2);
    }
     
    public static void main(String[] args) 
    {       
        Evaluation.Test_4_19();
        Evaluation.Test_4_20();
        Evaluation.Test_4_21();
        Evaluation.Test_4_22();
        Evaluation.Test_4_23();
        Evaluation.Test_4_25();
        Evaluation.Test_4_26();
        Evaluation.Test_4_8();
        Evaluation.Test_4_28();
        Evaluation.Test_4_30();
        Evaluation.Test_4_31();
        Evaluation.Test_4_32();
        
        Evaluation.Test_Outlier();
    }
}