/*
 * Clustering.java
 *
 * Copyright (c) 2011 Daniel Wayne Crabtree - All Rights Reserved
 * 
 * If you use this, please cite my PhD Thesis.
 
   @PHDTHESIS{IWS11,
       AUTHOR =  {Daniel Crabtree},
       TITLE =   {Techniques for Improving Web Search by Understanding Queries},
       SCHOOL =  {School of Engineering and Computer Science, Victoria University of Wellington},
       ADDRESS = {Wellington, New Zealand},
       MONTH =   {May},    
       YEAR =    {2011}
   }
  
 */

package qc4;

import java.util.*;

public class Clustering<T extends Cluster> implements Iterable<T>, Cloneable
{   
    private List<T> clustering;
    private ResultSet resultSet;
    
    public Clustering(ResultSet resultSet)
    {
        this.clustering = new ArrayList<T>();
        this.resultSet = resultSet;
    }
    
    public Object clone()
    {
        Clustering<T> object = null;
        
        try
        {
            object = (Clustering<T>)super.clone();
            object.clustering = new ArrayList<T>();
            for(T t: this.clustering)
                ((List<T>)object.clustering).add((T)t.clone());
        }
        catch(CloneNotSupportedException ex)
        {
            System.err.println("Clone not supported in Clustering");
        }
        
        return object;
    }
            
    public Iterator<T> iterator()
    {
        return this.clustering.iterator();
    }
    
    public final int size()
    {
        return this.clustering.size();
    }
            
    public final boolean add(T o)
    {
        return this.clustering.add(o);
    }
    
    public final ResultSet getResultSet()
    {
        return this.resultSet;
    }
    
    final List<T> clustersExcludingMostlyOutliers(IdealClustering idealClustering)
    {
        List<T> clustersExcludingMostlyOutliers = new ArrayList<T>();
        for(T cluster : this)
        {
            if(!cluster.isPredominantly(idealClustering.getOutlierCluster(), idealClustering.clustersExcludingOutlierOnLevel(1)))
                clustersExcludingMostlyOutliers.add(cluster);
        }
        return clustersExcludingMostlyOutliers;
    }
}
