/*
 * IdealCluster.java
 *
 * Copyright (c) 2011 Daniel Wayne Crabtree - All Rights Reserved
 * 
 * If you use this, please cite my PhD Thesis.
 
   @PHDTHESIS{IWS11,
       AUTHOR =  {Daniel Crabtree},
       TITLE =   {Techniques for Improving Web Search by Understanding Queries},
       SCHOOL =  {School of Engineering and Computer Science, Victoria University of Wellington},
       ADDRESS = {Wellington, New Zealand},
       MONTH =   {May},    
       YEAR =    {2011}
   }
  
 */

package qc4;

import java.util.*;

public class IdealCluster extends Cluster
{
    private List<IdealCluster> children;
    private int level;
    private boolean outlier;
    
    public IdealCluster(ResultSet resultSet)
    {
        this(resultSet, 1);
    }
    
    public IdealCluster(ResultSet resultSet, int level)
    {
        super(resultSet);
        this.children = new ArrayList<IdealCluster>();
        this.level = level;
        this.outlier = false;
    }
    
    public final void addChild(IdealCluster cluster)
    {
        if(!cluster.isSubCluster(this))
        {
            System.err.println("Cannot add child. The pages in the child ideal cluster are not a subset of the pages in the parent ideal cluster.");
            System.exit(0);
        }
        
        this.children.add(cluster);
    }
    
    public final void setOutlier()
    {
        this.outlier = true;
    }
    
    final boolean isOutlier()
    {
        return this.outlier;
    }    
    
    final int getLevel()
    {
        return this.level;
    }
    
    final boolean hasDescendant(IdealCluster cluster)
    {
        if(this.equals(cluster))
            return true;
        
        for(IdealCluster idealCluster: this.children)
        {
            if(idealCluster.hasDescendant(cluster))
                return true;
        }
        
        return false;
    }
}