/*
 * IdealClustering.java
 *
 * Copyright (c) 2011 Daniel Wayne Crabtree - All Rights Reserved
 * 
 * If you use this, please cite my PhD Thesis.
 
   @PHDTHESIS{IWS11,
       AUTHOR =  {Daniel Crabtree},
       TITLE =   {Techniques for Improving Web Search by Understanding Queries},
       SCHOOL =  {School of Engineering and Computer Science, Victoria University of Wellington},
       ADDRESS = {Wellington, New Zealand},
       MONTH =   {May},    
       YEAR =    {2011}
   }
  
 */

package qc4;

import java.util.*;

public class IdealClustering extends Clustering<IdealCluster>
{
    private Region regions;  
    
    public IdealClustering(ResultSet resultSet)
    {
        super(resultSet);
    }
       
    public final void setupRegions()
    {
        this.regions = new Region((IdealClustering)this.clone());
    }
    
    final List<Cluster> getRegions(int level)
    {
        return this.regions.get(level);
    }
     
    final List<IdealCluster> clustersOnLevel(int level)
    {
        List<IdealCluster> clustersOnLevel = new ArrayList<IdealCluster>();
        for(IdealCluster idealCluster : this)
        {
            if(idealCluster.getLevel() == level)
                clustersOnLevel.add(idealCluster);
        }
        return clustersOnLevel;
    }
    
    final List<IdealCluster> clustersExcludingOutlier()
    {
        List<IdealCluster> clustersExcludingOutlier = new ArrayList<IdealCluster>();
        for(IdealCluster idealCluster : this)
        {
            if(!idealCluster.isOutlier())
                clustersExcludingOutlier.add(idealCluster);
        }
        return clustersExcludingOutlier;
    }
    
    final List<IdealCluster> clustersExcludingOutlierOnLevel(int level)
    {
        List<IdealCluster> clustersExcludingOutlierOnLevel = new ArrayList<IdealCluster>();
        for(IdealCluster idealCluster : this)
        {
            if(idealCluster.getLevel() == level && !idealCluster.isOutlier())
                clustersExcludingOutlierOnLevel.add(idealCluster);
        }
        return clustersExcludingOutlierOnLevel;
    }
    
    final IdealCluster getOutlierCluster()
    {
        for(IdealCluster idealCluster: this)
        {
            if(idealCluster.isOutlier())
                return idealCluster;
        }
        
        return null;
    }
}