/*
 * Region.java
 *
 * Copyright (c) 2011 Daniel Wayne Crabtree - All Rights Reserved
 * 
 * If you use this, please cite my PhD Thesis.
 
   @PHDTHESIS{IWS11,
       AUTHOR =  {Daniel Crabtree},
       TITLE =   {Techniques for Improving Web Search by Understanding Queries},
       SCHOOL =  {School of Engineering and Computer Science, Victoria University of Wellington},
       ADDRESS = {Wellington, New Zealand},
       MONTH =   {May},    
       YEAR =    {2011}
   }
  
 */

package qc4;

import java.util.*;

class Region 
{
    private IdealClustering idealClustering;
    private Map<Integer, List<Cluster>> regionLevel;
    
    Region(IdealClustering idealClustering)
    {
        this.idealClustering = idealClustering;
        this.regionLevel = new HashMap<Integer, List<Cluster>>();
    }
    
    List<Cluster> get(int level)
    {
        if(!this.regionLevel.containsKey(level))
        {
            List<Cluster> regions = this.regions(level);
            this.regionLevel.put(level, regions);
        }
        
        return this.regionLevel.get(level);
    }
      
    private List<Cluster> regions(int level)
    {        
        List<IdealCluster> idealClustersOnLevel = this.idealClustering.clustersOnLevel(level);

        if(idealClustersOnLevel.isEmpty())
            return new ArrayList<Cluster>();
        
        Map<String, Cluster> regions = new HashMap<String, Cluster>();
        for(Page page : this.idealClustering.getResultSet())
        {
            String regionName = this.getPageRegion(page, idealClustersOnLevel);
            if(!regions.containsKey(regionName))
            {
                Cluster region = new Cluster(this.idealClustering.getResultSet());
                regions.put(regionName, region);
            }
            regions.get(regionName).addPage(page.getLocation());
        }
        
        return new ArrayList<Cluster>(regions.values());
    }
    
    private String getPageRegion(Page page, List<IdealCluster> idealClustersOnLevel)
    {
        StringBuilder regionName = new StringBuilder();
        for(int i = 0; i < idealClustersOnLevel.size(); i++)
        {
            if(idealClustersOnLevel.get(i).contains(page))
            {
                regionName.append("c").append(i);
            }
        }
        return regionName.toString();
    }
}